@@ -1,13 +1,12 @@
-import copy
 from dataclasses import is_dataclass
 from itertools import zip_longest
-from typing import TypeVar, Type, Optional, get_type_hints, Mapping, Any
+from typing import TypeVar, Type, Optional, get_type_hints, Mapping, Any, Collection, MutableMapping
 
+from dacite.cache import cache
 from dacite.config import Config
 from dacite.data import Data
 from dacite.dataclasses import (
     get_default_value_for_field,
-    create_instance,
     DefaultValueNotFoundError,
     get_fields,
     is_frozen,
@@ -28,10 +27,10 @@ from dacite.types import (
     is_union,
     extract_generic,
     is_optional,
-    transform_value,
     extract_origin_collection,
     is_init_var,
     extract_init_var,
+    is_subclass,
 )
 
 T = TypeVar("T")
@@ -45,36 +44,32 @@ def from_dict(data_class: Type[T], data: Data, config: Optional[Config] = None)
     :param config: a configuration of the creation process
     :return: an instance of a data class
     """
-    init_values: Data = {}
-    post_init_values: Data = {}
+    init_values: MutableMapping[str, Any] = {}
+    post_init_values: MutableMapping[str, Any] = {}
     config = config or Config()
     try:
-        data_class_hints = get_type_hints(data_class, globalns=config.forward_references)
+        data_class_hints = cache(get_type_hints)(data_class, localns=config.hashable_forward_references)
     except NameError as error:
         raise ForwardReferenceError(str(error))
-    data_class_fields = get_fields(data_class)
+    data_class_fields = cache(get_fields)(data_class)
     if config.strict:
         extra_fields = set(data.keys()) - {f.name for f in data_class_fields}
         if extra_fields:
             raise UnexpectedDataError(keys=extra_fields)
     for field in data_class_fields:
-        field = copy.copy(field)
-        field.type = data_class_hints[field.name]
+        field_type = data_class_hints[field.name]
         if field.name in data:
             try:
                 field_data = data[field.name]
-                transformed_value = transform_value(
-                    type_hooks=config.type_hooks, cast=config.cast, target_type=field.type, value=field_data
-                )
-                value = _build_value(type_=field.type, data=transformed_value, config=config)
+                value = _build_value(type_=field_type, data=field_data, config=config)
             except DaciteFieldError as error:
                 error.update_path(field.name)
                 raise
-            if config.check_types and not is_instance(value, field.type):
-                raise WrongTypeError(field_path=field.name, field_type=field.type, value=value)
+            if config.check_types and not is_instance(value, field_type):
+                raise WrongTypeError(field_path=field.name, field_type=field_type, value=value)
         else:
             try:
-                value = get_default_value_for_field(field)
+                value = get_default_value_for_field(field, field_type)
             except DefaultValueNotFoundError:
                 if not field.init:
                     continue
@@ -83,19 +78,32 @@ def from_dict(data_class: Type[T], data: Data, config: Optional[Config] = None)
             init_values[field.name] = value
         elif not is_frozen(data_class):
             post_init_values[field.name] = value
-
-    return create_instance(data_class=data_class, init_values=init_values, post_init_values=post_init_values)
+    instance = data_class(**init_values)
+    for key, value in post_init_values.items():
+        setattr(instance, key, value)
+    return instance
 
 
 def _build_value(type_: Type, data: Any, config: Config) -> Any:
     if is_init_var(type_):
         type_ = extract_init_var(type_)
+    if type_ in config.type_hooks:
+        data = config.type_hooks[type_](data)
+    if is_optional(type_) and data is None:
+        return data
     if is_union(type_):
-        return _build_value_for_union(union=type_, data=data, config=config)
-    elif is_generic_collection(type_) and is_instance(data, extract_origin_collection(type_)):
-        return _build_value_for_collection(collection=type_, data=data, config=config)
-    elif is_dataclass(type_) and is_instance(data, Data):
-        return from_dict(data_class=type_, data=data, config=config)
+        data = _build_value_for_union(union=type_, data=data, config=config)
+    elif is_generic_collection(type_):
+        data = _build_value_for_collection(collection=type_, data=data, config=config)
+    elif cache(is_dataclass)(type_) and isinstance(data, Mapping):
+        data = from_dict(data_class=type_, data=data, config=config)
+    for cast_type in config.cast:
+        if is_subclass(type_, cast_type):
+            if is_generic_collection(type_):
+                data = extract_origin_collection(type_)(data)
+            else:
+                data = type_(data)
+            break
     return data
 
 
@@ -108,12 +116,9 @@ def _build_value_for_union(union: Type, data: Any, config: Config) -> Any:
         try:
             # noinspection PyBroadException
             try:
-                data = transform_value(
-                    type_hooks=config.type_hooks, cast=config.cast, target_type=inner_type, value=data
-                )
+                value = _build_value(type_=inner_type, data=data, config=config)
             except Exception:  # pylint: disable=broad-except
                 continue
-            value = _build_value(type_=inner_type, data=data, config=config)
             if is_instance(value, inner_type):
                 if config.strict_unions_match:
                     union_matches[inner_type] = value
@@ -132,10 +137,10 @@ def _build_value_for_union(union: Type, data: Any, config: Config) -> Any:
 
 def _build_value_for_collection(collection: Type, data: Any, config: Config) -> Any:
     data_type = data.__class__
-    if is_instance(data, Mapping):
+    if isinstance(data, Mapping) and is_subclass(collection, Mapping):
         item_type = extract_generic(collection, defaults=(Any, Any))[1]
         return data_type((key, _build_value(type_=item_type, data=value, config=config)) for key, value in data.items())
-    elif is_instance(data, tuple):
+    elif isinstance(data, tuple) and is_subclass(collection, tuple):
         if not data:
             return data_type()
         types = extract_generic(collection)
@@ -144,5 +149,7 @@ def _build_value_for_collection(collection: Type, data: Any, config: Config) ->
         return data_type(
             _build_value(type_=type_, data=item, config=config) for item, type_ in zip_longest(data, types)
         )
-    item_type = extract_generic(collection, defaults=(Any,))[0]
-    return data_type(_build_value(type_=item_type, data=item, config=config) for item in data)
+    elif isinstance(data, Collection) and is_subclass(collection, Collection):
+        item_type = extract_generic(collection, defaults=(Any,))[0]
+        return data_type(_build_value(type_=item_type, data=item, config=config) for item in data)
+    return data
