@@ -1,7 +1,7 @@
 from dataclasses import Field, MISSING, _FIELDS, _FIELD, _FIELD_INITVAR  # type: ignore
 from typing import Type, Any, TypeVar, List
 
-from dacite.data import Data
+from dacite.cache import cache
 from dacite.types import is_optional
 
 T = TypeVar("T", bound=Any)
@@ -11,27 +11,23 @@ class DefaultValueNotFoundError(Exception):
     pass
 
 
-def get_default_value_for_field(field: Field) -> Any:
+@cache
+def get_default_value_for_field(field: Field, type_: Type) -> Any:
     if field.default != MISSING:
         return field.default
     elif field.default_factory != MISSING:  # type: ignore
         return field.default_factory()  # type: ignore
-    elif is_optional(field.type):
+    elif is_optional(type_):
         return None
     raise DefaultValueNotFoundError()
 
 
-def create_instance(data_class: Type[T], init_values: Data, post_init_values: Data) -> T:
-    instance = data_class(**init_values)
-    for key, value in post_init_values.items():
-        setattr(instance, key, value)
-    return instance
-
-
+@cache
 def get_fields(data_class: Type[T]) -> List[Field]:
     fields = getattr(data_class, _FIELDS)
     return [f for f in fields.values() if f._field_type is _FIELD or f._field_type is _FIELD_INITVAR]
 
 
+@cache
 def is_frozen(data_class: Type[T]) -> bool:
     return data_class.__dataclass_params__.frozen
